import json
import hashlib
from io import BytesIO
from RabbitSpider.http.request import Request
from urllib.parse import urlencode


class DupeFilter(object):
    def __init__(self, *args, **kwargs):
        pass

    def request_fingerprint(self, request: Request):
        if isinstance(request.data, (dict, list, tuple)):
            body = urlencode(request.data).encode('utf-8')
        elif isinstance(request.data, str):
            body = request.data.encode('utf-8')
        elif isinstance(request.data, BytesIO):
            body = request.data.read()
        elif isinstance(request.data, bytes):
            body = request.data
        else:
            body = b""

        if request.json is not None:
            body = json.dumps(request.json, separators=(",", ":")).encode()

        sha1 = hashlib.sha1()
        if isinstance(request.params, (dict, list, tuple)):
            sha1.update(f'{request.url}?{urlencode(request.params)}'.encode('utf-8'))
        else:
            sha1.update(request.url.encode('utf-8'))
        sha1.update(request.method.encode('utf-8'))
        sha1.update(body)
        sha1.update(str(request.retry).encode('utf-8'))
        return sha1.hexdigest()

    def request_seen(self, request: Request):
        pass



import asyncio
import os
import sys
from RabbitSpider.http.request import Request
from RabbitSpider.http.response import Response
from RabbitSpider.core.engine import Engine

if sys.platform == 'win32':
    asyncio.set_event_loop_policy(asyncio.WindowsSelectorEventLoopPolicy())

sys.path.append(os.path.abspath(os.path.join(os.path.abspath(sys.argv[0]), '..')))
sys.path.append(os.path.abspath(os.path.join(os.path.abspath(sys.argv[0]), '../..')))
sys.path.append(os.path.abspath(os.path.join(os.path.abspath(sys.argv[0]), '../../..')))
sys.path.append(os.path.abspath(os.path.join(os.path.abspath(sys.argv[0]), '../../../..')))



class Field(dict):
    pass


class ItemMeta(type):
    def __new__(mcs, name, bases, attrs):
        field = {}
        for key, value in attrs.items():
            if isinstance(value, Field):
                field[key] = value
        cls_instance = super().__new__(mcs, name, bases, attrs)
        cls_instance.FIELDS = field
        return cls_instance



class BasePipeline(object):
    def __init__(self, spider):
        pass

    async def open_spider(self, spider):
        """初始化数据库"""
        pass

    async def process_item(self, item, spider):
        """入库逻辑"""
        pass

    async def close_spider(self, spider):
        pass

    @classmethod
    def create_instance(cls, spider):
        return cls(spider)



import re
from typing import Callable, get_type_hints
from RabbitSpider.utils.exceptions import RabbitExpect


class Request(object):
    def __init__(self,
                 url: str,
                 params: dict = None,
                 data: dict | str | bytes | None = None,
                 json: dict = None,
                 method: str = 'get',
                 headers: dict | None = None,
                 cookies: dict | None = None,
                 proxy: str | None = None,
                 timeout: int | None = None,
                 allow_redirects: bool = True,
                 callback: str | Callable = 'parse',
                 retry: int = 0,
                 meta: dict | None = None
                 ):
        self.url = url
        self.params = params
        self.data = data
        self.json = json
        self.method = method
        self.headers = headers
        self.cookies = cookies
        self.proxy = proxy
        self.timeout = timeout
        self.allow_redirects = allow_redirects
        self._callback = callback
        self.retry = retry
        self._meta = meta
        self.__argsValidators__({k: v for k, v in locals().items() if k not in ['self']})

    def __argsValidators__(self, kwargs):
        pattern = re.compile('http[s]?://(?:[a-zA-Z]|[0-9]|[$-_@.&+]|[!*\(\),]|(?:%[0-9a-fA-F][0-9a-fA-F]))+')
        hints = get_type_hints(self.__init__)
        for key, value in kwargs.items():
            args_type = hints.get(key)
            if args_type:
                if not isinstance(value, args_type):
                    raise RabbitExpect(f"Request参数类型错误{key}")
            else:
                raise RabbitExpect(f"Request异常参数{key}")
        if not re.match(pattern, kwargs['url']):
            raise RabbitExpect(f'请检查url是否正确{kwargs["url"]}')

    @property
    def meta(self):
        return self._meta if self._meta else {}

    @property
    def callback(self):
        return self._callback.__name__ if callable(self._callback) else self._callback

    def to_dict(self):
        return {
            'url': self.url,
            'params': self.params,
            'data': self.data,
            'json': self.json,
            'method': self.method,
            'headers': self.headers,
            'cookies': self.cookies,
            'proxy': self.proxy,
            'timeout': self.timeout,
            'allow_redirects': self.allow_redirects,
            'callback': self.callback,
            'meta': self.meta,
            'retry': self.retry
        }



from RabbitSpider import Request
from RabbitSpider import Response


class BaseMiddleware:
    def __init__(self, spider):
        pass

    async def process_request(self, request, spider) -> None | Request | Response:
        """请求预处理"""
        pass

    async def process_response(self, request, response, spider) -> Request | Response:
        """响应预处理"""
        pass

    async def process_exception(self, request, exc, spider) -> None | Request | Response:
        """异常预处理"""
        pass

    @classmethod
    def create_instance(cls, spider):
        return cls(spider)



from RabbitSpider.items import ItemMeta


class BaseItem(metaclass=ItemMeta):
    def __init__(self):
        self._values = {}

    def __setitem__(self, key, value):
        if key in self.FIELDS:
            self._values[key] = value
        else:
            raise KeyError(f'field {key} undefined')

    def __getitem__(self, item):
        return self._values[item]

    def __iter__(self):
        return iter(self._values)

    def __len__(self) -> int:
        return len(self._values)

    def __delitem__(self, v):
        delattr(self._values, v)

    def __setattr__(self, key, value):
        if not key.startswith('_'):
            raise
        else:
            super().__setattr__(key, value)

    def __getattribute__(self, item):
        field = super().__getattribute__('FIELDS')
        if item in field:
            raise
        else:
            return super(BaseItem, self).__getattribute__(item)

    def to_dict(self):
        return self._values




from RabbitSpider.dupefilters import DupeFilter


class MemoryFilter(DupeFilter):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.repeat = set()

    def request_seen(self, request):
        fingerprint = self.request_fingerprint(request)
        if fingerprint in self.repeat:
            return False
        else:
            self.repeat.add(fingerprint)
            return True



import pickle
from typing import Callable, Optional
from aio_pika import connect_robust, Message


class Scheduler(object):
    def __init__(self, settings):
        self.username = settings.get('RABBIT_USERNAME')
        self.password = settings.get('RABBIT_PASSWORD')
        self.host = settings.get('RABBIT_HOST')
        self.port = settings.get('RABBIT_PORT')
        self.virtual_host = settings.get('RABBIT_VIRTUAL_HOST')

    async def connect(self):
        connection = await connect_robust(host=self.host, login=self.username, password=self.password,
                                          virtualhost=self.virtual_host)
        channel = await connection.channel()
        return connection, channel

    @staticmethod
    async def create_queue(channel, queue: str):
        await channel.declare_queue(name=queue, durable=True, arguments={"x-max-priority": 10})

    @staticmethod
    async def producer(channel, queue: str, body: dict):
        ret = pickle.dumps(body)
        await channel.default_exchange.publish(
            Message(body=ret, delivery_mode=2, priority=body['retry']), routing_key=queue)

    @staticmethod
    async def consumer(channel, queue: str, callback: Optional[Callable] = None, prefetch: int = 1):
        queue = await channel.declare_queue(name=queue, durable=True, passive=True, arguments={"x-max-priority": 10})
        if callback:
            await channel.set_qos(prefetch_count=prefetch)
            await queue.consume(callback=callback)
        else:
            return await queue.get()

    @staticmethod
    async def queue_purge(channel, queue: str):
        queue = await channel.declare_queue(name=queue, durable=True, passive=True, arguments={"x-max-priority": 10})
        await queue.purge()

    @staticmethod
    async def delete_queue(channel, queue: str):
        await channel.queue_delete(queue)

    @staticmethod
    async def get_message_count(channel, queue: str):
        queue = await channel.declare_queue(name=queue, durable=True, passive=True)
        return queue.declaration_result.message_count



import asyncio
import random

from RabbitSpider.middlewares import BaseMiddleware


class DownloadDelayMiddleware(BaseMiddleware):
    def __init__(self, spider):
        super().__init__(spider)
        self.download_delay = spider.settings.get('DOWNLOAD_DELAY')

    async def process_request(self, request, spider):
        if self.download_delay:
            delay = random.uniform(self.download_delay[0], self.download_delay[1])
            await asyncio.sleep(delay)



import asyncio
import sys
import os
import requests
import socket
from datetime import datetime
from typing import Type, List
from traceback import print_exc
from signal import signal, SIGINT, SIGTERM
from RabbitSpider.utils.control import TaskManager


async def main(spider, mode, task_count):
    try:
        rabbit = spider(task_count)
    except Exception:
        print_exc()
        raise

    # def signal_handler(sig, frame):
    #     requests.post('http://60.204.154.131:8000/post/task',
    #                   json={'status': 0, 'stop_time': datetime.now().strftime('%Y-%m-%d %H:%M:%S'), 'pid': os.getpid()})
    #
    # signal(SIGINT, signal_handler)
    # signal(SIGTERM, signal_handler)
    try:
        # requests.post('http://60.204.154.131:8000/post/task',
        #               json={'name': rabbit.name, 'ip_address': f'{socket.gethostbyname(socket.gethostname())}',
        #                     'task_count': task_count, 'status': 1, 'pid': os.getpid(), 'mode': mode,
        #                     'create_time': datetime.now().strftime('%Y-%m-%d %H:%M:%S'),
        #                     'dir': os.path.abspath(os.path.dirname(sys.argv[0]))})
        await rabbit.run(mode)

        # requests.post('http://60.204.154.131:8000/post/task',
        #               json={'pid': os.getpid(), 'name': rabbit.name, 'status': 2, 'mode': mode})

    except Exception:
        print_exc()


async def go(spider: Type, mode: str = 'auto', task_count: int = 1):
    for i in sys.argv[1:]:
        key, value = i.split('=')
        if key == 'mode':
            mode = value
        if key == 'task_count':
            task_count = int(value)
    await main(spider, mode=mode, task_count=task_count)


async def batch_go(spiders: List[Type], mode: str = 'auto', task_pool: int = 10):
    task_manager = TaskManager(task_pool)
    for spider in spiders:
        await task_manager.semaphore.acquire()
        task_manager.create_task(go(spider, mode=mode, task_count=1))
    while True:
        if task_manager.all_done():
            break
        else:
            await asyncio.sleep(1)



import os
import sys
import redis
from RabbitSpider.dupefilters import DupeFilter


class RedisFilter(DupeFilter):
    def __init__(self, settings):
        super().__init__(settings)
        self.repeat = os.path.basename(sys.argv[0])
        self.redis = redis.StrictRedis(host=settings.get('REDIS_HOST'), port=settings.get('REDIS_PORT'),
                                       db=settings.get('REDIS_DB'), password=settings.get('REDIS_PASSWORD'))
        self.redis.delete(self.repeat)

    def request_seen(self, request):
        fingerprint = self.request_fingerprint(request)
        if self.redis:
            result = self.redis.sadd(self.repeat, fingerprint)
            return result



from RabbitSpider.middlewares import BaseMiddleware


class AllowHttpCodeMiddleware(BaseMiddleware):
    def __init__(self, spider):
        super().__init__(spider)
        self.allow_http_code = spider.settings.getlist('ALLOW_HTTP_CODES')

    async def process_response(self, request, response, spider):
        if response.status not in self.allow_http_code:
            spider.logger.error(f'{request.to_dict()}，不允许的状态码：{response.status}')
            return True



import asyncio
from collections import defaultdict
from importlib import import_module
from inspect import iscoroutinefunction
from typing import Final, Dict, List, Callable
from asyncio import Task, Future, Semaphore
from RabbitSpider import Request
from RabbitSpider import Response
from RabbitSpider import default_settings
from RabbitSpider.core.download import CurlDownload


def load_class(_path):
    if not isinstance(_path, str):
        if callable(_path):
            return _path
        else:
            raise TypeError(f"args expected string or object, got {type(_path)}")
    module, name = _path.rsplit('.', 1)
    mod = import_module(module)
    try:
        cls = getattr(mod, name)
    except AttributeError:
        raise NameError(f"Module {module!r} doesn't define any project named {name!r}")

    return cls


async def call_function(func, *args, **kwargs):
    if iscoroutinefunction(func):
        return await func(*args, **kwargs)
    else:
        return func(*args, **kwargs)


class SettingManager(object):
    def __init__(self, custom_settings: dict):
        try:
            settings = import_module('settings')
        except ModuleNotFoundError:
            settings = default_settings
        self.attribute = {}
        for key in dir(settings):
            if key.isupper():
                self.attribute[key] = getattr(settings, key)
        self.attribute.update(custom_settings)

    def __setitem__(self, key, value):
        self.attribute[key] = value

    def __getitem__(self, key):
        return self.attribute.get(key)

    def __delitem__(self, key):
        del self.attribute[key]

    def get(self, key, value=None):
        return self[key] if self[key] else value

    def getlist(self, key):
        return self[key] if self[key] else []

    def set(self, key, value):
        self[key] = value


class TaskManager(object):
    def __init__(self, task_count: int):
        self.current_task: Final[set] = set()
        self.semaphore = Semaphore(task_count)

    def create_task(self, coroutine) -> Task:
        task = asyncio.create_task(coroutine)
        self.current_task.add(task)

        def done_callback(_fut: Future):
            self.current_task.remove(task)
            self.semaphore.release()

        task.add_done_callback(done_callback)
        return task

    def all_done(self):
        return len(self.current_task) == 0


class PipelineManager(object):
    def __init__(self, spider):
        self.spider = spider
        self.methods: Dict[str, List[Callable]] = defaultdict(list)
        self._add_pipe(spider.settings.getlist('ITEM_PIPELINES'))

    def _add_pipe(self, pipelines):
        for pipeline in pipelines:
            pipeline_obj = load_class(pipeline).create_instance(self.spider)
            if hasattr(pipeline_obj, 'open_spider'):
                self.methods['open_spider'].append(getattr(pipeline_obj, 'open_spider'))
            if hasattr(pipeline_obj, 'process_item'):
                self.methods['process_item'].append(getattr(pipeline_obj, 'process_item'))
            if hasattr(pipeline_obj, 'close_spider'):
                self.methods['close_spider'].append(getattr(pipeline_obj, 'close_spider'))

    async def open_spider(self):
        for method in self.methods['open_spider']:
            await call_function(method, self.spider)

    async def process_item(self, req):
        for method in self.methods['process_item']:
            await call_function(method, req, self.spider)

    async def close_spider(self):
        for method in self.methods['close_spider']:
            await call_function(method, self.spider)

    @classmethod
    def create_instance(cls, spider):
        return cls(spider)


class MiddlewareManager(object):
    def __init__(self, spider):
        self.spider = spider
        self.download = CurlDownload(spider.settings.get('HTTP_VERSION'), spider.settings.get('IMPERSONATE'))
        self.methods: Dict[str, List[Callable]] = defaultdict(list)
        self._add_middleware(spider.settings.getlist('MIDDLEWARES'))

    def _add_middleware(self, middlewares):
        for middleware in middlewares:
            middleware_obj = load_class(middleware).create_instance(self.spider)
            if hasattr(middleware_obj, 'process_request'):
                self.methods['process_request'].append(getattr(middleware_obj, 'process_request'))
            if hasattr(middleware_obj, 'process_response'):
                self.methods['process_response'].append(getattr(middleware_obj, 'process_response'))
            if hasattr(middleware_obj, 'process_exception'):
                self.methods['process_exception'].append(getattr(middleware_obj, 'process_exception'))

    async def _process_request(self, request):
        for method in self.methods['process_request']:
            result = await call_function(method, request, self.spider)
            if isinstance(result, (Request, Response)):
                return result
            if result:
                break
        else:
            return await self.download.fetch(self.spider.session, request.to_dict())

    async def _process_response(self, request, response):
        for method in reversed(self.methods['process_response']):
            result = await call_function(method, request, response, self.spider)
            if isinstance(result, (Request, Response)):
                return result
            if result:
                break
        else:
            return response

    async def _process_exception(self, request, exc):
        for method in self.methods['process_exception']:
            result = await call_function(method, request, exc, self.spider)
            if isinstance(result, (Request, Response)):
                return result
            if result:
                break
        else:
            raise exc

    async def downloader(self, request: Request):
        try:
            resp = await self._process_request(request)
        except Exception as exc:
            resp = await self._process_exception(request, exc)
        if isinstance(resp, Response):
            resp = await self._process_response(request, resp)
        if isinstance(resp, Request):
            resp = await self.spider.routing(resp)
        return request, resp

    @classmethod
    def create_instance(cls, spider):
        return cls(spider)


class FilterManager(object):
    def __init__(self, settings):
        filter_cls = settings.get('DUPEFILTER_CLASS')
        if filter_cls:
            self.filter_obj = load_class(filter_cls)(settings)
        else:
            self.filter_obj = None

    def request_seen(self, request: Request) -> bool:
        if self.filter_obj:
            result = self.filter_obj.request_seen(request)
            return result
        else:
            return True



import os
import shutil
import argparse
from string import Template
from RabbitSpider.utils.control import SettingManager

settings = SettingManager({})


def tmpl_file_path(_path):
    for i in os.listdir(_path):
        if os.path.isfile(os.path.join(_path, i)):
            if i.endswith('tmpl'):
                yield os.path.join(_path, i)
        if os.path.isdir(os.path.join(_path, i)):
            for i in tmpl_file_path(os.path.join(_path, i)):
                yield i


def template_to_file(_path, _dir, _file):
    if _path.lower() == 'test':
        print(f'项目名称不可以是{_path}')
        return
    try:
        shutil.copytree(settings.get('TEMPLATE_DIR'), _path)
    except FileExistsError:
        if not os.path.exists(os.path.join(_path, 'spiders', _dir)):
            os.mkdir(os.path.abspath(os.path.join(_path, 'spiders', _dir)))

        if os.path.exists(os.path.join(_path, 'spiders', _dir, f'{_file}.py')):
            print(f'{_path}/spiders/{_file}已存在')
            return
        shutil.copy(os.path.abspath(os.path.join(settings.get('TEMPLATE_DIR'), 'spiders/src/basic.tmpl')),
                    os.path.join(_path, 'spiders', _dir))
    for file in tmpl_file_path(_path):
        with open(file, 'r', encoding='utf-8') as f:
            text = Template(f.read()).substitute(bot_dir=os.path.abspath(f'./{_path}'), project=_path, dir=_dir,
                                                 spider=_file, classname=_file.capitalize())
        with open(file.replace('tmpl', 'py'), 'w', encoding='utf-8') as f:
            f.write(text)
        os.remove(file)
    if not os.path.exists(os.path.join(_path, 'spiders', _dir)):
        os.rename(os.path.abspath(os.path.join(_path, 'spiders', 'src')),
                  os.path.abspath(os.path.join(_path, 'spiders', _dir)))
    os.rename(os.path.join(_path, 'spiders', _dir, 'basic.py'),
              os.path.join(_path, 'spiders', _dir, f'{_file}.py'))
    print(f'{_path}/{_dir}/{_file}创建完成')


def create_project():
    parser = argparse.ArgumentParser()
    parser.add_argument('create', default='create', help='参数：create')
    parser.add_argument('project', help='参数：项目名称')
    parser.add_argument('dir', help='参数：目录')
    parser.add_argument('spider', help='参数：文件名称')
    args = parser.parse_args()
    template_to_file(f'{args.project}', _dir=f'{args.dir}', _file=f'{args.spider}')



from RabbitSpider.middlewares import BaseMiddleware


class RetryMiddleware(BaseMiddleware):
    def __init__(self, spider):
        super().__init__(spider)
        self.retry_http_code = spider.settings.getlist('RETRY_HTTP_CODES')
        self.retry_exceptions = spider.settings.getlist('RETRY_EXCEPTIONS')
        self.max_retry = spider.settings.get('MAX_RETRY')

    async def process_response(self, request, response, spider):
        if response.status in self.retry_http_code:
            if request.retry < self.max_retry:
                request.retry += 1
                return request
            else:
                spider.logger.warning(f'丢弃{request.to_dict()}，状态码：{response.status}')
                return True

    async def process_exception(self, request, exc, spider):
        if exc.__class__.__name__ in self.retry_exceptions:
            if request.retry < self.max_retry:
                request.retry += 1
                return request
            else:
                spider.logger.warning(f'丢弃{request.to_dict()}，异常：{repr(exc)}')
                return True



import setuptools

setuptools.setup(
    name='RabbitSpider',
    version='2.6.7',
    author='一纸',
    author_email='2395396520@qq.com',
    url='https://github.com/YunTom/RabbitSpider/tree/master',
    packages=['RabbitSpider', 'RabbitSpider.core', 'RabbitSpider.dupefilters', 'RabbitSpider.http',
              'RabbitSpider.items', 'RabbitSpider.middlewares', 'RabbitSpider.pipelines', 'RabbitSpider.utils'],
    include_package_data=True,
    entry_points={
        'console_scripts': [
            'rabbit = RabbitSpider.utils.template:create_project',
        ],
    },
    python_requires='>=3.10',
    install_requires=[
        'aio-pika>=9.4.1',
        'curl_cffi>=0.6.2',
        'loguru>=0.7.2',
        'parsel>=1.9.1',
        'redis>=5.0.3',
        'w3lib>=2.1.2',
        'chardet>=5.2.0',
        'loguru>=0.7.2',
        'chardet>=5.2.0'
    ],
)



import pickle
import asyncio
from traceback import print_exc
from aio_pika import IncomingMessage
from RabbitSpider.http.request import Request
from RabbitSpider.http.response import Response
from RabbitSpider.utils.control import TaskManager
from RabbitSpider.utils.control import SettingManager
from RabbitSpider.utils.control import PipelineManager
from RabbitSpider.utils.control import MiddlewareManager
from RabbitSpider.utils.control import FilterManager
from RabbitSpider.utils.exceptions import RabbitExpect
from RabbitSpider.utils.log import Logger
from RabbitSpider.items.item import BaseItem
from RabbitSpider.core.scheduler import Scheduler
from collections.abc import AsyncGenerator, Coroutine, Generator
from aio_pika.exceptions import QueueEmpty, ChannelClosed, ChannelNotFoundEntity


class Engine(object):
    name: str
    custom_settings: dict = {}

    def __init__(self, task_count):
        self.session = None
        self.__connection = None
        self.__channel = None
        self.__task_count = task_count
        self.settings = SettingManager(self.custom_settings)
        self.__scheduler = Scheduler(self.settings)
        self.__filter = FilterManager(self.settings)
        self.__task_manager = TaskManager(self.__task_count)
        self.__middlewares = MiddlewareManager.create_instance(self)
        self.__pipelines = PipelineManager.create_instance(self)
        self.logger = Logger(self.settings, self.name)

    async def start_requests(self):
        """初始请求"""
        pass

    async def parse(self, request: Request, response: Response):
        """默认回调"""
        pass

    async def routing(self, result):
        async def rule(res):
            if isinstance(res, Request):
                if self.__filter.request_seen(res):
                    self.logger.info(f'生产数据：{res.to_dict()}')
                    await self.__scheduler.producer(self.__channel, queue=self.name, body=res.to_dict())
            elif isinstance(res, BaseItem):
                await self.__pipelines.process_item(res)

        if isinstance(result, AsyncGenerator):
            async for r in result:
                await rule(r)
        elif isinstance(result, Generator):
            for r in result:
                await rule(r)
        elif isinstance(result, Coroutine):
            r = await result
            await rule(r)
        elif isinstance(result, Request):
            await rule(result)
        elif result is None:
            pass
        else:
            raise RabbitExpect('回调函数返回类型错误！')

    async def produce(self):
        try:
            await self.__scheduler.queue_purge(self.__channel, self.name)
            await self.routing(self.start_requests())
        except Exception as e:
            self.logger.error(f'{e}')
            print_exc()
            for task in asyncio.all_tasks():
                task.cancel()

    async def crawl(self):
        while True:
            try:
                incoming_message: IncomingMessage = await self.__scheduler.consumer(self.__channel, self.name)
            except QueueEmpty:
                if self.__task_manager.all_done():
                    await self.__scheduler.delete_queue(self.__channel, self.name)
                    break
                else:
                    continue
            except ChannelNotFoundEntity:
                break
            except ChannelClosed:
                await asyncio.sleep(1)
                self.logger.warning('rabbitmq重新连接')
                self.__connection, self.__channel = await self.__scheduler.connect()
                continue
            await self.__task_manager.semaphore.acquire()
            self.__task_manager.create_task(self.deal_resp(incoming_message))

    async def consume(self):
        while True:
            try:
                await self.__scheduler.consumer(self.__channel, queue=self.name, callback=self.deal_resp,
                                                prefetch=self.__task_count)
            except ChannelClosed:
                await asyncio.sleep(1)
                self.logger.warning('rabbitmq重新连接')
                self.__connection, self.__channel = await self.__scheduler.connect()
                continue
            await asyncio.Future()

    async def deal_resp(self, incoming_message: IncomingMessage):
        ret = pickle.loads(incoming_message.body)
        self.logger.info(f'消费数据：{ret}')
        request, response = await self.__middlewares.downloader(Request(**ret))
        if response:
            try:
                callback = getattr(self, ret['callback'])
                result = callback(request, response)
                if result:
                    await self.routing(result)
            except Exception as e:
                self.logger.error(f'解析失败：{e}')
                print_exc()
                for task in asyncio.all_tasks():
                    task.cancel()
            else:
                await incoming_message.ack()

    async def run(self, mode):
        self.logger.info(f'{self.name}任务开始')
        self.__connection, self.__channel = await self.__scheduler.connect()
        self.session = await self.__middlewares.download.new_session()
        await self.__scheduler.create_queue(self.__channel, self.name)
        await self.__pipelines.open_spider()
        if mode == 'auto':
            await self.produce()
            await self.crawl()
        elif mode == 'm':
            await self.produce()
        elif mode == 'w':
            await self.consume()
        else:
            raise RabbitExpect('执行模式错误！')
        await self.__pipelines.close_spider()
        await self.__channel.close()
        await self.__connection.close()
        await self.__middlewares.download.exit(self.session)
        self.logger.info(f'{self.name}任务完成')



import os
from loguru import logger


class Logger(object):
    def __init__(self, settings, name):
        log_path = settings.get('LOG_FILE')
        if log_path:
            if log_path.startswith('.'):
                log_path = os.path.join(settings.get('BOT_DIR'), log_path)
            logger.add("%s/rabbit_{time:YYYY-MM-DD}.log" % log_path,
                       level=settings.get('LOG_LEVEL', 'ERROR'),
                       rotation="1 day",
                       retention="1 week",
                       format="{time:YYYY-MM-DD HH:mm:ss} | {level} | {extra[scope]} | {name}:{line} - {message}")
            self.logger = logger.bind(scope=name)
        else:
            self.logger = logger

    def info(self, msg):
        self.logger.info(msg)

    def warning(self, msg):
        self.logger.warning(msg)

    def error(self, msg):
        self.logger.error(msg)



import json as js
import chardet
import parsel
from w3lib.encoding import http_content_type_encoding, html_to_unicode


class Response:
    def __init__(self, res):
        self.body = self.content = res.content
        self.status = self.status_code = res.status_code
        self.charset = res.charset
        self.headers = {k: v for k, v in res.headers.items()}
        self.cookies = {k: v for k, v in res.cookies.items()}
        self.__r = parsel.Selector(self.text)

    def __str__(self):
        return self.text

    @property
    def text(self):
        if not self.content:
            return ''
        if self.charset:
            try:
                text = self.content.decode(self.charset)
            except UnicodeDecodeError:
                try:
                    benc = http_content_type_encoding(self.headers['Content-Type'])
                    if benc:
                        charset = 'charset=%s' % benc
                        text = html_to_unicode(charset, self.body)[1]
                    else:
                        raise UnicodeDecodeError
                except UnicodeDecodeError:
                    try:
                        char = chardet.detect(self.content)
                        if char:
                            text = self.content.decode(char['encoding'])
                        else:
                            raise UnicodeDecodeError
                    except UnicodeDecodeError:
                        try:
                            text = self.content.decode('utf-8')
                        except UnicodeDecodeError:
                            try:
                                text = self.content.decode("gb18030")
                            except UnicodeDecodeError:
                                text = self.content.decode('utf-8', "ignore")
        else:
            try:
                text = self.content.decode('utf-8')
            except UnicodeDecodeError:
                try:
                    char = chardet.detect(self.content)
                    if char:
                        text = self.content.decode(char['encoding'])
                    else:
                        raise UnicodeDecodeError
                except UnicodeDecodeError:
                    try:
                        text = self.content.decode('gb18030')
                    except UnicodeDecodeError:
                        text = self.content.decode('utf-8', "ignore")
        return text

    def json(self):
        return js.loads(self.text, strict=False)

    def xpath(self, x):
        return self.__r.xpath(x)

    def css(self, x):
        return self.__r.css(x)



from curl_cffi.requests import AsyncSession
from RabbitSpider import Response
from RabbitSpider.utils.exceptions import RabbitExpect


class CurlDownload(object):
    def __init__(self, http_version, impersonate):
        self.impersonate = impersonate
        self.http_version = http_version

    @staticmethod
    async def new_session():
        session = AsyncSession(verify=False)
        return session

    @staticmethod
    async def exit(session):
        await session.close()

    async def fetch(self, session, request):
        if request['method'].upper() == 'GET':
            res = await session.get(request['url'],
                                    params=request.get('params', None), cookies=request.get('cookies', None),
                                    headers=request.get('headers', None), proxy=request.get('proxy', None),
                                    allow_redirects=request.get('allow_redirects', True),
                                    http_version=self.http_version,
                                    impersonate=self.impersonate,
                                    timeout=request.get('timeout', 60)
                                    )

        elif request['method'].upper() == 'POST':
            res = await session.post(request['url'],
                                     data=request.get('data', None), json=request.get('json', None),
                                     cookies=request.get('cookies', None), headers=request.get('headers', None),
                                     proxy=request.get('proxy', None),
                                     http_version=self.http_version,
                                     impersonate=self.impersonate,
                                     allow_redirects=request.get('allow_redirects', True),
                                     timeout=request.get('timeout', 180))

        else:
            raise RabbitExpect(f"{request['method']}请求方式未定义，请自定义添加！")

        if res:
            response = Response(res)
            return response
